#include "PhysicalObjects/arena.h"
#include "mathutility.h"
#include "drawutilities.h"
#include "world.h"

#include <QJsonObject>
#include <QRectF>
#include <QBrush>
#include <QPen>
#include <QPainter>

const QString Arena::Name = QString("Arena");

PhysicalObject *arenaFactory(World *world, const QJsonObject &obj)
{

    QJsonObject positionObj = obj["position"].toObject();
    QPointF position = QPointF(positionObj["x"].toDouble(), positionObj["y"].toDouble());
    float width = obj["width"].toDouble();
    float height = obj["height"].toDouble();

    //arena->CreatePhysicalBody(position, width, height, world->world());
    Arena *arena = new Arena(world->world(), position, width, height);
    return arena;
}

Arena::Arena(b2World *world, QPointF position, float width, float height) :
    PhysicalObject(), QGraphicsItem(), _boundingRect(QPointF(-width - 0.5f, -height - 0.5f), QPointF(width + 0.5f, height + 0.5f))
{
    CreatePhysicalBody(position, width, height, world);

    setPos(position);
}

Arena::~Arena()
{

}

QRectF Arena::boundingRect() const
{
    return _boundingRect;
}

void Arena::paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget)
{
    genericPaint(painter, option, widget, _body);

    QPen p(Qt::red);
    p.setWidthF(0.1f);
    painter->setPen(p);
    painter->drawRect(_boundingRect);
}

void Arena::CreatePhysicalBody(QPointF position, float width, float height, b2World *world)
{
    b2BodyDef bodyDef;
    bodyDef.angle = 0;
    bodyDef.type = b2_staticBody;
    bodyDef.position.Set(position.x(), position.y());

    _body = world->CreateBody(&bodyDef);

    // Left wall
    b2PolygonShape wall1Shape;
    b2Vec2 wall1Center(-width, 0);
    wall1Shape.SetAsBox(0.5f, height, wall1Center, 0);
    _body->CreateFixture(&wall1Shape, 0);

    // Top wall
    b2PolygonShape wall2Shape;
    b2Vec2 wall2Center(0, -height);
    wall2Shape.SetAsBox(width, 0.5f, wall2Center, 0);
    _body->CreateFixture(&wall2Shape, 0);

    // Right wall
    b2PolygonShape wall3Shape;
    b2Vec2 wall3Center(width, 0);
    wall3Shape.SetAsBox(0.5f, height, wall3Center, 0);
    _body->CreateFixture(&wall3Shape, 0);

    // Bottom wall
    b2PolygonShape wall4Shape;
    b2Vec2 wall4Center(0, height);
    wall4Shape.SetAsBox(width, 0.5f, wall4Center, 0);
    _body->CreateFixture(&wall4Shape, 0);
}
