#include "chirpnetworkcontroller.h"

#include <QUdpSocket>
#include <QByteArray>
#include <QTextStream>

#include "robotrepository.h"
#include "PhysicalObjects/chirp.h"

#define SETSPEED 0
#define GETSENSORS 1

ChirpNetworkController::ChirpNetworkController(Chirp *robot)
{
    _robot = robot;
    _acceptedMessages.resize(2);
    _acceptedMessages[SETSPEED] = "setspeed";
    _acceptedMessages[GETSENSORS] = "getsensors";
}

QByteArray ChirpNetworkController::handleMessage(QTextStream &stream)
{
    try
    {
        QString type;
        stream >> type;

        switch(_acceptedMessages.indexOf(type)) {
        case SETSPEED: // "setspeed" message
            return handleSetSpeedMessage(stream);

        case GETSENSORS: // "getsensors" message
            return handleGetSensorsMessage();

        default:
            return QString("error").toUtf8();
        }
    }
    catch(...)
    {
        return QString("error").toUtf8();
    }
}

QByteArray ChirpNetworkController::handleSetSpeedMessage(QTextStream &stream)
{
    float leftWheel, rightWheel;
    QVector<float> speeds;
    stream >> leftWheel >> rightWheel;
    speeds << leftWheel << rightWheel;
    _robot->setSpeed(speeds);

    return QByteArray("ok");
}

QByteArray ChirpNetworkController::handleGetSensorsMessage()
{
    QVector<float> sensors = _robot->sensors();
    QString result = QString("ok ");
    for(int i = 0; i < sensors.size(); ++i)
    {
        result += QString("%1 ").arg(sensors[i]);
    }
    return result.trimmed().toUtf8();
}
