 #include "controllerserver.h"

#include <QUdpSocket>
#include <QMessageBox>
#include <QByteArray>
#include <QTextStream>
#include <QHostAddress>

#include "robotrepository.h"
#include "robot.h"

ControllerServer::ControllerServer(QObject *parent, quint16 port) :
    QObject(parent)
{
    _server = new QUdpSocket(this);

    _port = port;
}

ControllerServer::~ControllerServer()
{
}

void ControllerServer::start()
{
    if(!_server->bind(_port))
    {
        QMessageBox::critical(0, "ControllerServer", _server->errorString());
        exit(-1);
    }

    connect(_server, SIGNAL(readyRead()), this, SLOT(incomingConnection()));
}

void ControllerServer::incomingConnection()
{
    while(_server->hasPendingDatagrams()) {
        QByteArray datagram;
        QHostAddress sender;
        quint16 port;
        datagram.resize(_server->pendingDatagramSize());
        _server->readDatagram(datagram.data(), _server->pendingDatagramSize(), &sender, &port);
        QTextStream textStream(&datagram); // Using stream to consume id and send the remaining message to controller.
        unsigned id;
        textStream >> id;

        Robot *robot = RobotRepository::instance()->getRobotFromId(id);
        QByteArray response;
        if(robot == NULL)
        {
            response = QByteArray("error");
        }
        else
        {
            response = robot->controller()->handleMessage(textStream);
        }
        _server->writeDatagram(response.data(), sender, port);
    }
}
