#ifndef PHYSICALOBJECT_H
#define PHYSICALOBJECT_H

#include <Box2D/Box2D.h>
#include <QGraphicsItem>

/*!
 * \brief The PhysicalObject class is an abstract base class giving access to the required attributes of the physical object.
 *
 * The PhysicalObject class is an abstract base class giving access to the required attributes of the physical object.
 * It also provides a default implementation of the step function that does nothing.
 */
class PhysicalObject
{
public:
    PhysicalObject() {}
    virtual ~PhysicalObject() {}

    /*!
     * \brief Called by World on all PhysicalObjects in the simulation so they can implement any custom physics.
     * \param dt seconds passed since last call.
     */
    virtual void step(float dt) { Q_UNUSED(dt); }

    /*!
     * \brief stepGraphics
     */
    virtual void stepGraphics() {}

    /*!
     * \brief Simple getter to get the physical representation of an object.
     * \return Physical representation of object used by b2World class.
     */
    virtual b2Body *body() const = 0;

    /*!
     * \brief Simple getter for returning QGraphicsItem connected to this object.
     * \return The graphic representation of the object.
     */
    virtual QGraphicsItem *item() const = 0;
};

#endif // PHYSICALOBJECT_H
