#ifndef CONTROLLER_H
#define CONTROLLER_H

#include <QObject>
#include <QTextStream>

/*!
 * \brief The Controller class is an interface class that controller interfaces for robots should implement.
 *
 * The handleMessage-method should be overridden by concrete controller interfaces.
 */
class Controller : public QObject
{
    Q_OBJECT
public:
    Controller(QObject* parent=0) : QObject(parent) {}
    virtual ~Controller() {}

    /*!
     * \brief handleMessage is a pure virtual function controllers overload to handle any message sent their way.
     * \param textStream is a stream containing the message to handle.
     * \return A utf-8 string containing either "ok" + any result, or "error" to signify something failed.
     */
    virtual QByteArray handleMessage(QTextStream &textStream)=0;
};

#endif // NETWORKCONTROLLER_H
