#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include "simulatorview.h"

namespace Ui {
class MainWindow;
}

/*!
 * \brief The MainWindow class is the main window for the application, containing a SimulatorView along with supporting menu items for ease of use.
 */
class MainWindow : public QMainWindow
{
    Q_OBJECT
    
public:
    explicit MainWindow(QWidget *parent = 0);
    ~MainWindow();

public slots:
    /*!
     * \brief Called when the SimulatorView detects a change in which item is selected.
     *
     * Connected to the signal selectionChanged of QGraphicsView.
     */
    void selectedItemChanged(void);
    
private slots:
    /*!
     * \brief Closes the main window and exits the program. Called when the Exit menu item is clicked.
     */
    void on_actionExit_triggered();

    /*!
     * \brief Zooms in the SimulatorView. Called when the Zoom In menu item is clicked.
     */
    void on_actionZoom_in_triggered();

    /*!
     * \brief Zooms out the SimulatorView. Called when the Zoom Out menu item is clicked.
     */
    void on_actionZoom_out_triggered();

    /*!
     * \brief Loads a new world from a JSON-file and creates a new SimulatorView to view the world. Called when the Load World menu item is clicked.
     */
    void on_actionLoad_World_triggered();

    /*!
     * \brief Sets the simulation to real-time.
     */
    void on_action1x_triggered();

    /*!
     * \brief Sets the simulation to 2x real-time.
     */
    void on_action2x_triggered();

    /*!
     * \brief Sets the simulation to 3x real-time.
     */
    void on_action3x_triggered();

private: // Methods
    void createSimulatorView();
    void destroySimulatorView();

private:
    Ui::MainWindow *ui;

    SimulatorView* simulatorView;
};

#endif // MAINWINDOW_H
