#pragma once
#ifndef MATHUTILITY_H
#define MATHUTILITY_H

#define DEGTORAD 0.0174532925199432957f
#define RADTODEG 57.295779513082320876f
#define PI 3.14159265359f

#include <QPointF>
#include <QList>
#include <Box2D/Box2D.h>

/*!
 * \brief Gets the sign of the floating point value.
 * \param x Value to get the sign of.
 * \return -1 if x < 0, 0 if x == 0, 1 if x > 0.
 */
inline int sign(float x)
{
    if(x > 0) return 1;
    return (x < 0) ? -1 : 0;
}

/*!
 * \brief Returns the minimal value of x and y.
 * \param x First value.
 * \param y Second value.
 * \return If x <= y returns x. If y < x returns y.
 */
inline float min(float x, float y)
{
    return (x <= y) ? x : y;
}

/*!
 * \brief Returns the maximal value of x and y.
 * \param x First value.
 * \param y Second value.
 * \return If x > y returns x. If y >= x returns y.
 */
inline float max(float x, float y)
{
    return (x > y) ? x : y;
}

/*!
 * \brief Clamps x between minValue and maxValue.
 * \param x Value to be clamped.
 * \param minValue Minimum value x can have.
 * \param maxValue Maximum value x can have.
 * \return minValue if x < minValue. x if minValue < x < maxValue. maxValue if x > maxvalue.
 */
inline float clamp(float x, float minValue, float maxValue)
{
    return min(maxValue, max(minValue, x));
}

/*!
 * \brief Converts an array of b2Vec2 into an array of QPointF
 * \param points Array of b2Vec2 to convert.
 * \param num_points Number of points in the array.
 * \param result Array to store the result in.
 *
 * \note Assumes that result is an array equal or larger than the points array.
 */
void b2Vec2ToQPointF(b2Vec2* points, size_t num_points, QPointF* result);

/*!
 * \brief Converts a list of QPointF into an array of b2Vec2.
 * \param points Source list to convert.
 * \param result Array to store the result in.
 *
 * \note Assumes that result is an array equal or larger than the points array.
 */
void QPointFTob2Vec2(QList<QPointF> points, b2Vec2* result);

#endif // MATHUTILITY_H
