#ifndef PHYSICALOBJECTFACTORY_H
#define PHYSICALOBJECTFACTORY_H

#include <QObject>
#include <QMap>
#include <QJsonObject>
#include "PhysicalObjects/physicalobject.h"
#include "world.h"

/*!
 * \brief FactoryFunction is a function pointer taking in a World object and a JsonObject and returns a pointer to a PhysicalObject.
 * \return Pointer to a constructed PhysicalObject.
 */
typedef PhysicalObject* FactoryFunction(World *, const QJsonObject&);

/*!
 * \brief The PhysicalObjectFactory class is a singleton class allowing users to register factories for a PhysicalObject class through a type name.
 */
class PhysicalObjectFactory : public QObject
{
    Q_OBJECT


public:
    /*!
     * \brief addFactory registers a root object factory on a type name.
     * \param name type name of the object.
     * \param factoryFunction factory function used to create the object from a json object.
     */
    void addFactory(const QString& name, FactoryFunction factoryFunction);

    /*!
     * \brief getFactory
     * \param name type name of the object.
     * \return Factory function for creating an object of type "name" from a json object.
     */
    FactoryFunction *getFactory(const QString& name);

    /*!
     * \return singleton instance of the class.
     */
    static PhysicalObjectFactory *instance();
signals:
    
public slots:
    
private:
    explicit PhysicalObjectFactory();

    static PhysicalObjectFactory *_instance;

    QMap<QString, FactoryFunction*> _map;
};

#endif // PHYSICALOBJECTFACTORY_H
