#ifndef ROBOT_H
#define ROBOT_H

#include <QVector>
#include <QSharedPointer>
#include "controller.h"

/*!
 * \brief The Robot class is the abstract base class for robots, containing an implementation for setting a controller, id, and giving the
 *        interface for getting sensor values and setting the wheel speeds of the robot.
 */
class Robot {
public:
    /*!
     * \param wheelSpeeds list of wheel speeds for the robot.
     */
    virtual void setSpeed(QVector<float> wheelSpeeds)=0;

    /*!
     * \return A list of all sensor values from the robot.
     */
    virtual QVector<float> sensors() const=0;

    /*!
     * \return ID of the robot.
     */
    unsigned id() const { return _id; }

    /*!
     * \return The controller attached to the robot.
     */
    Controller *controller() { return _controller; }

protected:
    void setId(unsigned id) { _id = id; }
    void setController(Controller *controller) { _controller = controller; }
private:
    unsigned _id;

    Controller *_controller;
};

#endif // ROBOT_H
