#ifndef ROBOTREPOSITORY_H
#define ROBOTREPOSITORY_H

#include <QObject>
#include <QMap>
#include <QSharedPointer>

class Robot;

/*!
 * \brief The RobotRepository class is a singleton class storing each Robot in the simulation.
 *
 * The class allows others to store a robot into the repository, then get it by supplying the unique ID of the robot.
 */
class RobotRepository
{
public:
    /*!
     * \brief Static getter method to get the singleton instance of the class.
     * \return Singleton instance of RobotRepository.
     */
    static RobotRepository* instance();

    /*!
     * \brief Stores a robot in the repository.
     * \param robot The robot to store.
     */
    void addRobot(Robot *robot);

    /*!
     * \brief Gets the robot with the unique ID supplied.
     * \param id Unique ID for the robot.
     * \return Robot instance with the ID supplied.
     */
    Robot *getRobotFromId(unsigned id);

protected:
    explicit RobotRepository();
    
signals:
    
public slots:
    
private:
    static RobotRepository* _instance;

    QMap<quint32, Robot*> _map;
};

#endif // ROBOTREPOSITORY_H
