#ifndef SENSOR_H
#define SENSOR_H

#include "PhysicalObjects/physicalobject.h"

/*!
 * \brief The Sensor class is the interface used by sensors. It contains methods to update the sensor,
 *        get the value of the sensor and get the PhysicalObject the sensor can detect.
 */
class Sensor
{
public:
    /*!
     * \brief Gets the value of the sensor.
     * \return The value of the sensor.
     */
    virtual unsigned value() = 0;

    /*!
     * \brief Updates the value of the sensor.
     */
    virtual void update() = 0;

    /*!
     * \brief Gets the PhysicalObject sensed by the sensor.
     * \return The PhysicalObject sensed by the sensor.
     */
    virtual PhysicalObject *objectHit() = 0;
};

#endif // SENSOR_H
