#ifndef SIMULATORVIEW_H
#define SIMULATORVIEW_H

#include <QGraphicsView>
#include <QElapsedTimer>
#include <QTimer>
#include <QSharedPointer>
#include <QMap>

#include <Box2D/Box2D.h>

#include "world.h"
#include "controllerserver.h"

/*!
 * \brief The SimulatorView class implements a view of the World currently being simulated.
 */
class SimulatorView : public QGraphicsView
{
    Q_OBJECT
public:
    explicit SimulatorView(QWidget *parent = 0);
    ~SimulatorView();

    /*!
     * \brief Called when the mouse wheel moves inside the SimulatorView. Calls zoom(1) when the wheel is scrolled upwards and zoom(-1) when the wheel is scrolled downwards.
     * \param event QWheelEvent sent by the QGraphicsView base class.
     */
    void wheelEvent(QWheelEvent *event);

    /*!
     * \brief Called when the mouse moves within SimulatorView. Will scroll around the world when the right mouse button is held down.
     * \param event QMouseEvent sent by the QGraphicsView base class.
     */
    void mouseMoveEvent(QMouseEvent *event);

    /*!
     * \brief Zooms the view in or out depending on the sign of direction.
     * \param direction A positive value will zoom in, a negative value will zoom out.
     */
    void zoom(int direction);

    /*!
     * \brief Sets the number of physical steps being run between each time the graphics are updated.
     * \param numberOfSteps Number of steps between each time the graphics are updated.
     */
    void setPhysicalSteps(unsigned numberOfSteps);
signals:
    
public slots:
    /*!
     * \brief Called by a QTimer periodically, depending on the set timestep.
     */
    void step(void);

private:
    World *_world;

    QTimer *_timer;
    QElapsedTimer *_elapsedTimer;

    unsigned _timestep; // milliseconds per step.
    unsigned _physicalSteps;
};

#endif // SIMULATORVIEW_H
