#include "mainwindow.h"
#include "simulatorview.h"
#include "ui_mainwindow.h"

#include "PhysicalObjects/chirp.h"
#include "controllerserver.h"

#include <QList>
#include <QGraphicsItem>
#include <QDebug>
#include <QThread>
#include <QFileDialog>
#include <QString>
#include <QTextStream>
#include <QFile>
#include <QJsonDocument>

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);
    createSimulatorView();

    QThread* controllerThread = new QThread(this);
    ControllerServer* controllerServer = new ControllerServer(0);
    controllerServer->moveToThread(controllerThread);
    connect(this, SIGNAL(destroyed()), controllerThread, SLOT(quit()));
    connect(controllerThread, SIGNAL(started()), controllerServer, SLOT(start()));
    connect(controllerThread, SIGNAL(finished()), controllerServer, SLOT(deleteLater()));

    controllerThread->start();
}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::selectedItemChanged()
{
    QList<QGraphicsItem*> selectedItems = simulatorView->scene()->selectedItems();
    for(QList<QGraphicsItem*>::const_iterator it = selectedItems.constBegin(); it != selectedItems.constEnd(); ++it)
    {
        Chirp *bot = static_cast<Chirp*>(*it);
        qDebug() << bot->id();
        qDebug() << (*it)->pos().x() << "," << (*it)->pos().y();
    }
}

void MainWindow::on_actionExit_triggered()
{
    this->close();
}

void MainWindow::on_actionZoom_in_triggered()
{
    simulatorView->zoom(1);
}

void MainWindow::on_actionZoom_out_triggered()
{
    simulatorView->zoom(-1);
}

void MainWindow::on_actionLoad_World_triggered()
{
    destroySimulatorView();

    QString filename = QFileDialog::getOpenFileName();
    QFile file(filename);
    file.open(QIODevice::ReadOnly | QIODevice::Text);
    QTextStream in(&file);
    QString worldJson = in.readAll();
    file.close();
    QJsonDocument doc = QJsonDocument::fromJson(worldJson.toUtf8());
    World::fromJsonDocument(doc);

    createSimulatorView();
}

void MainWindow::createSimulatorView()
{
    simulatorView = new SimulatorView(this);
    ui->verticalLayout->addWidget(simulatorView, 1);
    connect(simulatorView->scene(), SIGNAL(selectionChanged()), this, SLOT(selectedItemChanged()));
}

void MainWindow::destroySimulatorView()
{
    disconnect(simulatorView->scene(), SIGNAL(selectionChanged()), this, SLOT(selectedItemChanged()));
    ui->verticalLayout->removeWidget(simulatorView);
    delete simulatorView;
    simulatorView = NULL;
}

void MainWindow::on_action1x_triggered()
{
    simulatorView->setPhysicalSteps(1);
}

void MainWindow::on_action2x_triggered()
{
    simulatorView->setPhysicalSteps(2);
}

void MainWindow::on_action3x_triggered()
{
    simulatorView->setPhysicalSteps(3);
}
