#include "simulatorview.h"

#include <QGraphicsScene>
#include <QGraphicsEllipseItem>
#include <QGraphicsItem>
#include <QGLWidget>
#include <QFileDialog>
#include <QDebug>
#include <QMouseEvent>
#include <QScreen>
#include <QWheelEvent>
#include <QSharedPointer>
#include <cmath>

#include "PhysicalObjects/arena.h"
#include "PhysicalObjects/chirp.h"
#include "PhysicalObjects/wall.h"
#include "mathutility.h"
#include "controllerserver.h"
#include "robot.h"
#include "robotrepository.h"

#include "PhysicalObjects/physicalobject.h"

SimulatorView::SimulatorView(QWidget *parent) :
    QGraphicsView(parent)
{
    setCacheMode(QGraphicsView::CacheBackground);
    setViewportUpdateMode(QGraphicsView::MinimalViewportUpdate);
    setRenderHint(QPainter::Antialiasing);

    _physicalSteps = 1;

    _world = World::instance();

    _world->setSceneRect(-1000, -1000, 2000, 2000);
    setScene(_world);

    _timer = new QTimer(this);
    _elapsedTimer = new QElapsedTimer();

    connect(_timer, SIGNAL(timeout()), this, SLOT(step()));

    _timestep = 16;

    _timer->setSingleShot(true);

    _elapsedTimer->start();
    _timer->start(_timestep);
}

SimulatorView::~SimulatorView()
{
}

void SimulatorView::wheelEvent(QWheelEvent *event)
{
    zoom(sign(event->delta()));
}

void SimulatorView::mouseMoveEvent(QMouseEvent *event)
{
    static QPointF lastPos(mapToGlobal(rect().center()));
    if(event->buttons() & Qt::RightButton)
    {
        QPointF dPos = event->screenPos() - lastPos;
        centerOn(mapToScene((width() * 0.5f) + dPos.x(), (height() * 0.5f) + dPos.y()));

        QCursor::setPos(mapToGlobal(rect().center()));
    }
}

void SimulatorView::zoom(int direction)
{
    float zoom = 0.25f * sign(direction) + 1;
    scale(zoom, zoom);
}

void SimulatorView::setPhysicalSteps(unsigned numberOfSteps)
{
    _physicalSteps = numberOfSteps;
}

void SimulatorView::step()
{
    _elapsedTimer->restart();
    for(unsigned i = 0; i < _physicalSteps; ++i)
        _world->step(_timestep / 1000.0f);

    _world->stepGraphics();
    int timeElapsed = _elapsedTimer->restart();
    int timeToNext = _timestep - timeElapsed;
    timeToNext = (timeToNext > 0) ? timeToNext : 0;
    _timer->start(timeToNext);
}
