#include "world.h"

#include <QJsonObject>
#include <QJsonArray>
#include <QJsonDocument>

#include <Box2D/Box2D.h>

#include "PhysicalObjects/physicalobject.h"
#include "physicalobjectfactory.h"

World *World::_instance = NULL;

World::World() : QGraphicsScene(0)
{
    _world = new b2World(b2Vec2(0, 0));
}

World::~World()
{
    delete _world;
    _instance = NULL;
}

void World::addObject(PhysicalObject *object)
{
    _physicalObjects.append(object);
    addItem(object->item());
}

void World::removeObject(PhysicalObject *object)
{
    _physicalObjects.removeOne(object);
    _world->DestroyBody(object->body());
    removeItem(object->item());
    delete object;
}

void World::step(float dt)
{
    _world->Step(dt, 5, 5);

    foreach(PhysicalObject *object, _physicalObjects)
    {
        object->step(dt);
    }
}

void World::stepGraphics()
{
    foreach(PhysicalObject *object, _physicalObjects)
    {
        object->stepGraphics();
    }
}


World *World::instance()
{
    if(_instance == NULL)
        _instance = new World();
    return _instance;
}


World *World::fromJsonDocument(const QJsonDocument &doc)
{
    if(_instance != NULL)
    {
        delete _instance;
    }
    _instance = new World();

    QJsonObject obj = doc.object();
    QJsonArray children = obj["children"].toArray();
    foreach (const QJsonValue &childValue, children)
    {
        QJsonObject child = childValue.toObject();
        QString name = child["type"].toString().toLower();
        PhysicalObject *po = PhysicalObjectFactory::instance()->getFactory(name)(_instance, child);
        _instance->addObject(po);
    }

    return _instance;
}


b2World *World::world()
{
    return _world;
}
